import {
  Form, LoadingService, Modal, ModalService, SelectMultiple,
} from '@zeedhi/common';
import { TekGrid, TekTreeGrid } from '@zeedhi/teknisa-components-common';
import { Http, IEventParam, Metadata } from '@zeedhi/core';
import { Instances } from '@/types/InterfaceStoreBlockController';
import { alertService } from '@/helpers/AlertService';

interface ParamsBlockProduct {
  CDFILIAL: string,
  CDLOJA: string,
  CDPRODUTO: string[],
  CDOPERADOR: string | null,
  NRDIASEMANABLOQ: string[] | string,
}

export class ProductBlockController {
  private productBlockModal!: Modal;

  private keyPressed: string = '';

  public destroyProductBlockModal() {
    this.productBlockModal?.destroy();
  }

  public async openProductBlockModal() {
    if (this.productBlockModal) this.productBlockModal.destroy();
    this.productBlockModal = await ModalService.createFromJson('product-block-modal', true);
    await this.productBlockModal.show();
  }

  public closeProductBlockModal() {
    this.productBlockModal.destroy();
  }

  public grid(): TekTreeGrid {
    return Metadata.getInstance<TekTreeGrid>('product-block-grid');
  }

  public getInstance<T extends Instances>(name: string): T {
    const instance = Metadata.getInstance<T>(name);
    return instance;
  }

  public async handleCellClick({ component, row }: IEventParam<TekGrid>): Promise<void> {
    await this.rowClick(component, row);
  }

  public async rowClick(component: any, row : any): Promise<void> {
    try {
      const grid = this.grid();
      if (this.keyPressed === 'control') {
        component.selectRow(row, true);
        component.selectChildren(row);
      } else if (this.keyPressed === 'shift') {
        component.selectRow(row, false);
        component.unselectChildren(row);
      } else if (row.NMPRODUTO) {
        const productIndex = grid.selectedRows.findIndex((selectedProducts) => selectedProducts.CDFILIAL
          === row.CDFILIAL);
        if (productIndex > -1) {
          component.selectRow(row, false);
        } else {
          component.selectRow(row, true);
        }
      } else if (row.NMLOJA) {
        await this.openProductBlockModal();
        const form = this.getInstance<Form>('product-block-form');
        const select = this.getInstance<SelectMultiple>('product');
        if (form && select) {
          form.value = {
            CDFILIALGRID: row.CDFILIALGRID,
            CDFILIAL: row.CDFILIAL,
            PARENTFILIAL: row.PARENTFILIAL,
          };
          LoadingService.show();
          const productData = await this.fetchProductData(row.CDFILIAL, row.CDFILIALGRID);
          select.updateData(productData);
        }
      }
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    } finally {
      LoadingService.hide();
    }
  }

  public async fetchProductData(CDFILIAL: string, CDLOJA: string): Promise<any> {
    try {
      const { data: { data, error, message } } = await Http.post('/control/branch/availableProductsRepository',
        { CDFILIAL, CDLOJA });

      if (error) throw new Error(message || 'Erro na solicitação da API');
      if (data) return data || [];
      return [];
    } catch (error) {
      if (error instanceof Error) {
        throw new Error(error.message);
      }
      return [];
    }
  }

  public isValidFormBlockProduct(): boolean {
    const form = this.getInstance<Form>('product-block-form');
    const valid = form.validate();
    return valid;
  }

  public paramsBlockProductModal(): ParamsBlockProduct {
    const form = this.getInstance<Form>('product-block-form');
    const { value } = form;
    const cdOperador = localStorage.getItem('ID');
    return {
      CDFILIAL: value.PARENTFILIAL,
      CDLOJA: value.CDFILIALGRID,
      CDPRODUTO: value.product,
      CDOPERADOR: cdOperador,
      NRDIASEMANABLOQ: value.week.length === 7 ? 'T' : value.week,
    };
  }

  public async blockProduct(): Promise<any> {
    try {
      const isValid = this.isValidFormBlockProduct();
      if (!isValid) return;

      const params = this.paramsBlockProductModal();
      const grid = this.grid();

      LoadingService.show();
      const { data: { error, message } } = await Http.post('/control/branch/blockProducts', params);

      if (error) throw new Error(message || 'Erro na solicitação da API');

      alertService('success', message, 'green');
      Array.from(grid.selectedRows).forEach((product) => {
        grid.selectRow(product, false);
      });
      grid.reload();
      this.closeProductBlockModal();
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    } finally {
      LoadingService.hide();
    }
  }

  public async unblockProduct(): Promise<any> {
    try {
      const grid = this.grid();
      if (grid.selectedRows.length > 0) {
        const params: any = [];
        Array.from(grid.selectedRows).forEach((product) => {
          grid.selectRow(product, false);
          params.push({
            CDFILIAL: product.PARENTFILIAL,
            CDLOJA: product.PARENTSTORE,
            CDPRODUTO: product.CDFILIALGRID,
            NRDIASEMANABLOQ: product.NRDIASEMANABLOQ,
          });
        });
        LoadingService.show();
        const { data: { error, message } } = await Http.post('/control/branch/unblockProducts', params);

        if (error) throw new Error(message || 'Erro na solicitação da API');

        alertService('success', message, 'green');
        grid.reload();
      } else {
        alertService('error', 'Selecione algum produto para desbloquear.', 'red');
      }
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    } finally {
      LoadingService.hide();
    }
  }

  public mountProductBlockPage() {
    document.addEventListener('keydown', (event) => {
      if (event.key === 'Control') {
        this.keyPressed = 'control';
      } else if (event.key === 'Shift') {
        this.keyPressed = 'shift';
      }
    });
    document.addEventListener('keyup', (event) => {
      if (event.key === 'Control') {
        this.keyPressed = '';
      } else if (event.key === 'Shift') {
        this.keyPressed = '';
      }
    });
  }

  public async product(): Promise<void> {
    try {
      const grid = this.grid();
      LoadingService.show();
      const { data: { data, message, error } } = await Http.get('/control/branch/productBlock');

      if (error) throw new Error(message);

      grid.datasource.updateData(data);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    } finally {
      LoadingService.hide();
    }
  }
}
