/* eslint-disable */
import { Form, LoadingService, Modal, ModalService, SelectMultiple, Text } from '@zeedhi/common';
import { TekTreeGrid } from '@zeedhi/teknisa-components-common';
import { Http, IEventParam, Metadata } from '@zeedhi/core';
import { Instances } from '@/types/InterfaceStoreBlockController';
import { alertService } from '@/helpers/AlertService';

interface Row {
  CDFILIALGRID: string,
  CDFILIAL: string,
  NMLOJA: string,
  NMOPERADOR: string,
  DTHRBLOQUEIO: string,
  NRDIASEMANABLOQ: string,
  PARENTID: string,
  LABELNRDIASEMANA: string,
}
interface ParamsBlockStore {
  CDFILIAL: string | string[],
  CDLOJA: string[],
  CDOPERADOR: string | null,
  NRDIASEMANABLOQ: string[] | string,
}
export class StoreBlockController {
  private storeBlockModal!: Modal;

  public async createStoreBlockModal(): Promise<void> {
    this.storeBlockModal = await ModalService.createFromJson('store-block-modal', true);
  }

  public destroyStoreBlockModal(): void {
    this.storeBlockModal?.destroy();
  }

  public openStoreBlockModal(): void {
    this.storeBlockModal.show();
  }

  public async closeStoreBlockModal(): Promise<void> {
    const form = await this.getInstance<Form>('store-block-form');
    form.clearForm();
    this.storeBlockModal.hide();
  }

  public async mounted(): Promise<void> {
    await this.store();
  }

  public async store() {
    try {
      LoadingService.show();
      const grid = this.grid();

      const { data: { data, message, error } } = await Http.get('/branch/storeBlock');

      if (error) throw new Error(message);

      grid.datasource.updateData(data);
    } catch (error) {
      if (error instanceof Error)
        alertService('error', error.message, 'red');
    } finally {
      LoadingService.hide();
    }
  }

  public grid(): TekTreeGrid {
    return Metadata.getInstance<TekTreeGrid>('store-block-grid');
  }

  public async getInstance<T extends Instances>(name: string): Promise<T> {
    const instance: T = await new Promise((resolve) => {
      setTimeout(() => {
        resolve(Metadata.getInstances(name)[0]);
      }, 0);
    });
    return instance;
  }

  public async openStoreModal({ row }: IEventParam<TekTreeGrid>): Promise<void> {
    if (row.FILIAL === null || row.PARENTID) {
      this.openStoreBlockModal();
      await this.getUnit(row);
    }
  }

  public async getUnit({ CDFILIALGRID }: Row): Promise<void> {
    try {
      const select = Metadata.getInstance<SelectMultiple>('store');
      const textCdFilial = await this.getInstance<Text>('CDFILIAL');
      if (select && textCdFilial) {
        textCdFilial.text = CDFILIALGRID;
        const storeData = await this.fetchStoreData(CDFILIALGRID);
        select.datasource.updateData(storeData);
      }
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public async fetchStoreData(CDFILIAL: string): Promise<any> {
    try {
      const { data: { data, error, message } } =
        await Http.post('/branch/availableStoresRepository', { CDFILIAL });

      if (error) throw new Error(message || 'Erro na solicitação da API');
      if (data) return data || [];
    } catch (error) {
      if (error instanceof Error) {
        throw new Error(error.message);
      }
    }
  }

  public async isValidFormBlockStore(): Promise<boolean> {
    const form = await this.getInstance<Form>('store-block-form');
    const valid = form.validate();
    return valid;
  }

  public async paramsBlockStoreModal(): Promise<ParamsBlockStore> {
    const form = await this.getInstance<Form>('store-block-form');
    const { text } = await this.getInstance<Text>('CDFILIAL');
    const { value } = form;
    const cdOperador = localStorage.getItem('ID');
    return {
      CDFILIAL: text,
      CDLOJA: value.store,
      CDOPERADOR: cdOperador,
      NRDIASEMANABLOQ: value.week.length === 7 ? 'T' : value.week,
    };
  }

  public async blockStore(): Promise<any> {
    try {
      const isValid = await this.isValidFormBlockStore();
      if (!isValid) return;

      const params = await this.paramsBlockStoreModal();

      LoadingService.show();
      const { data: { error, message } } = await Http.post('branch/blockStores', params);

      if (error) throw new Error(message || 'Erro na solicitação da API');

      alertService('success', message, 'green');
      this.closeStoreBlockModal();
      await this.store();
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    } finally {
      LoadingService.hide();
    }
  }
}